@@ -98,13 +98,13 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     QT_STATBUF buff_dest;
     bool dest_exists = (QT_LSTAT(_dest.data(), &buff_dest) != -1);
     if (dest_exists) {
-        if ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_DIR) {
-            error(KIO::ERR_DIR_ALREADY_EXIST, dest);
+        if (same_inode(buff_dest, buff_src)) {
+            error(KIO::ERR_IDENTICAL_FILES, dest);
             return;
         }
 
-        if (same_inode(buff_dest, buff_src)) {
-            error(KIO::ERR_IDENTICAL_FILES, dest);
+        if ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_DIR) {
+            error(KIO::ERR_DIR_ALREADY_EXIST, dest);
             return;
         }
 
@@ -196,7 +196,7 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
                 }
             } else
 #endif
-                error(KIO::ERR_COULD_NOT_READ, src);
+                error(KIO::ERR_CANNOT_READ, src);
             src_file.close();
             dest_file.close();
 #if HAVE_POSIX_ACL
@@ -218,7 +218,7 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
                     remove(_dest.data());
                 } else {
                     qWarning() << "Couldn't write[2]. Error:" << dest_file.errorString();
-                    error(KIO::ERR_COULD_NOT_WRITE, dest);
+                    error(KIO::ERR_CANNOT_WRITE, dest);
                 }
 #if HAVE_POSIX_ACL
                 if (acl) {
@@ -239,7 +239,7 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
 
     if (dest_file.error() != QFile::NoError) {
         qWarning() << "Error when closing file descriptor[2]:" << dest_file.errorString();
-        error(KIO::ERR_COULD_NOT_WRITE, dest);
+        error(KIO::ERR_CANNOT_WRITE, dest);
 #if HAVE_POSIX_ACL
         if (acl) {
             acl_free(acl);
@@ -394,7 +394,7 @@ void FileProtocol::listDir(const QUrl &url)
             listEntry(entry);
 
         } else {
-            if (createUDSEntry(filename, QByteArray(ep->d_name), entry, details, true)) {
+            if (createUDSEntry(filename, QByteArray(ep->d_name), entry, details)) {
                 listEntry(entry);
             }
         }
@@ -431,13 +431,13 @@ void FileProtocol::rename(const QUrl &srcUrl, const QUrl &destUrl,
     // with its target (#169547)
     bool dest_exists = (QT_LSTAT(_dest.data(), &buff_dest) != -1);
     if (dest_exists) {
-        if ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_DIR) {
-            error(KIO::ERR_DIR_ALREADY_EXIST, dest);
+        if (same_inode(buff_dest, buff_src)) {
+            error(KIO::ERR_IDENTICAL_FILES, dest);
             return;
         }
 
-        if (same_inode(buff_dest, buff_src)) {
-            error(KIO::ERR_IDENTICAL_FILES, dest);
+        if ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_DIR) {
+            error(KIO::ERR_DIR_ALREADY_EXIST, dest);
             return;
         }
 
@@ -534,7 +534,7 @@ void FileProtocol::del(const QUrl &url, bool isfile)
                 error(KIO::ERR_ACCESS_DENIED, path);
             } else {
                 // qDebug() << "could not rmdir " << perror;
-                error(KIO::ERR_COULD_NOT_RMDIR, path);
+                error(KIO::ERR_CANNOT_RMDIR, path);
                 return;
             }
         }
@@ -617,7 +617,7 @@ void FileProtocol::stat(const QUrl &url)
     const int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
 
     UDSEntry entry;
-    if (!createUDSEntry(url.fileName(), _path, entry, details, true /*with acls*/)) {
+    if (!createUDSEntry(url.fileName(), _path, entry, details)) {
         error(KIO::ERR_DOES_NOT_EXIST, path);
         return;
     }
