@@ -27,6 +27,7 @@
 #include <kio/copyjob.h>
 #include <kio/job.h>
 #include <kio/mkdirjob.h>
+#include <kio/mkpathjob.h>
 #include <kjobwidgets.h>
 #include <klocalizedstring.h>
 #include <kmessagebox.h>
@@ -152,12 +153,14 @@ CommandRecorder::CommandRecorder(FileUndoManager::CommandType op, const QList<QU
     connect(job, SIGNAL(result(KJob*)),
             this, SLOT(slotResult(KJob*)));
 
-    // TODO whitelist, instead
-    if (op != FileUndoManager::Mkdir && op != FileUndoManager::Put) {
+    if (qobject_cast<KIO::CopyJob *>(job)) {
         connect(job, SIGNAL(copyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)),
                 this, SLOT(slotCopyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)));
         connect(job, SIGNAL(copyingLinkDone(KIO::Job*,QUrl,QString,QUrl)),
                 this, SLOT(slotCopyingLinkDone(KIO::Job*,QUrl,QString,QUrl)));
+    } else if (KIO::MkpathJob *mkpathJob = qobject_cast<KIO::MkpathJob *>(job)) {
+        connect(mkpathJob, &KIO::MkpathJob::directoryCreated,
+                this, &CommandRecorder::slotDirectoryCreated);
     }
 }
 
@@ -174,7 +177,7 @@ void CommandRecorder::slotResult(KJob *job)
     FileUndoManager::self()->d->addCommand(m_cmd);
 }
 
-void CommandRecorder::slotCopyingDone(KIO::Job *job, const QUrl &from, const QUrl &to, const QDateTime &mtime, bool directory, bool renamed)
+void CommandRecorder::slotCopyingDone(KIO::Job *, const QUrl &from, const QUrl &to, const QDateTime &mtime, bool directory, bool renamed)
 {
     BasicOperation op;
     op.m_valid = true;
@@ -183,17 +186,6 @@ void CommandRecorder::slotCopyingDone(KIO::Job *job, const QUrl &from, const QUr
     op.m_src = from;
     op.m_dst = to;
     op.m_mtime = mtime;
-
-    if (m_cmd.m_type == FileUndoManager::Trash) {
-        Q_ASSERT(to.scheme() == "trash");
-        const QMap<QString, QString> metaData = job->metaData();
-        QMap<QString, QString>::ConstIterator it = metaData.find("trashURL-" + from.path());
-        if (it != metaData.constEnd()) {
-            // Update URL
-            op.m_dst = QUrl(it.value());
-        }
-    }
-
     m_cmd.m_opStack.prepend(op);
 }
 
@@ -211,6 +203,18 @@ void CommandRecorder::slotCopyingLinkDone(KIO::Job *, const QUrl &from, const QS
     m_cmd.m_opStack.prepend(op);
 }
 
+void CommandRecorder::slotDirectoryCreated(const QUrl &dir)
+{
+    BasicOperation op;
+    op.m_valid = true;
+    op.m_type = BasicOperation::Directory;
+    op.m_renamed = false;
+    op.m_src = QUrl();
+    op.m_dst = dir;
+    op.m_mtime = QDateTime();
+    m_cmd.m_opStack.prepend(op);
+}
+
 ////
 
 class KIO::FileUndoManagerSingleton
@@ -313,6 +317,8 @@ QString FileUndoManager::undoText() const
         return i18n("Und&o: Trash");
     case FileUndoManager::Mkdir:
         return i18n("Und&o: Create Folder");
+    case FileUndoManager::Mkpath:
+        return i18n("Und&o: Create Folder(s)");
     case FileUndoManager::Put:
         return i18n("Und&o: Create File");
     }
@@ -342,24 +348,27 @@ void FileUndoManager::undo()
     UndoCommand cmd = d->m_commands.last();
     assert(cmd.m_valid);
     d->m_current = cmd;
+    const CommandType commandType = cmd.m_type;
 
     BasicOperation::Stack &opStack = d->m_current.m_opStack;
     // Note that opStack is empty for simple operations like Mkdir.
 
     // Let's first ask for confirmation if we need to delete any file (#99898)
-    QList<QUrl> fileCleanupStack;
+    QList<QUrl> itemsToDelete;
     BasicOperation::Stack::Iterator it = opStack.begin();
     for (; it != opStack.end(); ++it) {
         BasicOperation::Type type = (*it).m_type;
-        if (type == BasicOperation::File && d->m_current.m_type == FileUndoManager::Copy) {
-            fileCleanupStack.append((*it).m_dst);
+        if (type == BasicOperation::File && commandType == FileUndoManager::Copy) {
+            itemsToDelete.append((*it).m_dst);
+        } else if (commandType == FileUndoManager::Mkpath) {
+            itemsToDelete.append((*it).m_dst);
         }
     }
-    if (d->m_current.m_type == FileUndoManager::Mkdir || d->m_current.m_type == FileUndoManager::Put) {
-        fileCleanupStack.append(d->m_current.m_dst);
+    if (commandType == FileUndoManager::Mkdir || commandType == FileUndoManager::Put) {
+        itemsToDelete.append(d->m_current.m_dst);
     }
-    if (!fileCleanupStack.isEmpty()) {
-        if (!d->m_uiInterface->confirmDeletion(fileCleanupStack)) {
+    if (!itemsToDelete.isEmpty()) {
+        if (!d->m_uiInterface->confirmDeletion(itemsToDelete)) {
             return;
         }
     }
@@ -404,13 +413,13 @@ void FileUndoManager::undo()
         }
     }
 
-    if (d->m_current.m_type == FileUndoManager::Put) {
+    if (commandType == FileUndoManager::Put) {
         d->m_fileCleanupStack.append(d->m_current.m_dst);
     }
 
     //qDebug() << "starting with" << undoStateToString(d->m_undoState);
     d->m_undoJob = new UndoJob(d->m_uiInterface->showProgressInfo());
-    d->undoStep();
+    QMetaObject::invokeMethod(d, "undoStep", Qt::QueuedConnection);
 }
 
 void FileUndoManagerPrivate::stopUndo(bool step)
